UNIT FTPSERV;
(*
**
** Copyright (C) 1999-2003 Maarten Bekers. All rights reserved.
**
** This file is part of EleBBS, EleXer and EleWEB.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*)
{$I COMPILER.INC}
(*
**
** FTPSERV, FTP server (RFC-959) for EleBBS.
**
** Copyright (c) 2000 by Maarten Bekers
**
** Created : 05-May-2000
** Last update : 11-Feb-2002
**
*)

(*
214-The following  commands are recognized(* ==>'s unimplemented).
   ABOR
   ALLO
   APPE
   CDUP
   CWD
   DELE
   HELP
   LIST
   MDTM
   MKD
   MODE
   NLST
   NOOP
   PASS
   PASV
   PORT
   PWD
   QUIT
   REST
   RETR
   RMD
   RNFR
   RNTO
   SITE
   SMNT
   STAT
   STOR
   STRU
   SYST
   TYPE
   USER
214  HELP command successful.
*)


(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)
 INTERFACE
(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

uses TcpSrv;

const
  ftp_ListenThreadEnded  : Boolean = false;

var
 FtpServerSocket: TTcpServer;

procedure ftp_SetupServer;
procedure ftp_initserver;

function ftp_ServerThread(P: pointer): Longint;
function ftp_ExecuteConnection(P: Pointer): Longint;

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)
 IMPLEMENTATION
(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

uses Crt, Global, CfgRec, CentrStr, ScrnU, LongStr, StUtils,
      GenCfg, JDates, IoRes_Un, SysUtils, ReadCfg,
       WinTitle, GenFile, MemMan, Cases, WordStr, Crc_Unit, BbsKey,
        FileRout, Mail, MkMsgAbs, MkOpen, SockFunc, StrPath, SockDef,
          Access_U, TcpCli, Threads, Windows, Ellog_U,
          FileObj, BitWise, Debug_U, Aptimer, Cfgfile,
           ChrIso, User_U, SysVars, ObjDec, Ra2Fdb, esrv_u;

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

const DebugSize         : Record
                            X1, Y1, X2, Y2: Byte;
                          end = (X1: 03; Y1: 02; X2:77; Y2:11);

type FtpCommandset = (ftpUser, ftpPass, ftpRetr, ftpError, ftpHelp,
                      ftpQuit, ftpPort, ftpList, ftpAllo,  ftpMkd,
                      ftpMode, ftpNlst, ftpNoop, ftpPwd,   ftpSmnt,
                      ftpCdUp, ftpCwd,  ftpStru, ftpType,  ftpSyst,
                      ftpOpts, ftpRnFr, ftpRnTo, ftpSite,  ftpDele,
                      ftpStat, ftpRmd,  ftpAppe, ftpRest,  ftpAbor,
                      ftpPasv);


Const FtpServerPort      = 21;
      FtpMaxSessions     = 10;

      SecsTimeOut            = 10 * 60;   { Timeout after 10 minutes inactivity }

      ServerName             : String = 'EleBBS - FTP server running at %s';

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

var ServerTID    : TThreadsObj;
    InputTID     : TThreadsObj;
    TitleStr     : String;
    UpdateTimer  : Eventtimer;

type
  ftpConnectionRec = record
                          IpAddr      : String[36];
                          DataThread  : TTcpClient;
                          FtpSession  : TTcpClient;
                          Validated   : Boolean;
                          UserName    : ShortString;
                          UserRec     : Longint;
                          PasswordCRC : Longint;
                          RemAddr     : Array[1..6] of Byte;
                          IsPassive   : Boolean;
                          Exitinfo    : ExitinfoRecord;
                          UseDataConn : Boolean;
                          CurGroup    : Longint;
                          CurArea     : Longint;
                        end; { record }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)


function FtpStatus(Code: Longint): String;
begin
  Result := '';

  Case Code of
    200 : ; { we cannot send 200 as it can mean many things }
    220 : Result := Format(ServerName, [GlobalCfg^.RaConfig^.SystemName]) + ' ready.';
    221 : Result := 'Goodbye.';
    226 : Result := 'Transfer complete';
    425 : Result := 'Can''t open data connection.';
    450 : Result := 'Requested file action not taken';
    500 : Result := 'Uknown command';
    502 : Result := 'Command not implemented';
    550 : Result := 'Requested action not taken.';
  end; { case }
end; { func. FtpStatus }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure SendCodeStr(const CurSlot: Longint; Code: Word);
begin
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    FtpSession.SendStrLn(Format('%d %s', [Code, FtpStatus(Code)]));
end; { proc. SendCodeStr }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure SendCodeMsg(const CurSlot: Longint; Code: Word; MsgStr: String);
begin
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    FtpSession.SendStrLn(Format('%d %s', [Code, MsgStr]));
end; { proc. SendCodeMsg }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function CloseDataConnection(const CurSlot: Longint; DidSuccess: Boolean): Boolean;
begin
  {-- initialize basic variables -------------------------------------------}
  CloseDataConnection := TRUE;

  {-- if we use the current connection, just do that -----------------------}
  if ftpConnectionRec(Connections[CurSlot].SrvData^).UseDataConn then
   with ftpConnectionRec(Connections[Curslot].SrvData^) do
     begin
       CloseDataConnection := TRUE;
       DataThread.SockHandle := -1;
     end; { if }


  {-- close down the data connection ---------------------------------------}
  if (NOT ftpConnectionRec(Connections[CurSlot].SrvData^).UseDataConn) AND
      (NOT ftpConnectionRec(Connections[CurSlot].SrvData^).IsPassive) then
        with ftpConnectionRec(Connections[Curslot].SrvData^) do
          begin
            DataThread.Disconnect;

            {-- let remote know we closed this connection ------------------}
            if DidSuccess then
              SendCodeStr(CurSlot, 226)
          end; { with }

end; { func. CloseDataConnection }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function OpenDataConnection(const CurSlot: Longint; var ResStr: String): Boolean;
var IpStr   : String;
    PortAr  : Array[1..2] of Byte;
    PortNum : Smallword ABSOLUTE PortAr;
begin
  {-- initialize basic variables -------------------------------------------}
  OpenDataConnection := FALSE;
  ResStr := '';

  {-- if we use the current connection, just do that -----------------------}
  if ftpConnectionRec(Connections[CurSlot].SrvData^).UseDataConn then
   with ftpConnectionRec(Connections[Curslot].SrvData^) do
     begin
       OpenDataConnection := TRUE;
       DataThread.SockHandle := FtpSession.SockHandle;
     end; { if }

  {-- we havent implemented PASSIVE mode yet -------------------------------}
  {-- so we connect to the specified ports ---------------------------------}
  if (NOT ftpConnectionRec(Connections[CurSlot].SrvData^).UseDataConn) AND
      (NOT ftpConnectionRec(Connections[CurSlot].SrvData^).IsPassive) then
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    begin
      {-- Make sure our data is in the specified form ----------------------}
      Move(RemAddr, IpAddr, 4);

      {-- move the last 2 bytes (ports) to the word-var --------------------}
      PortAr[1] := RemAddr[6];
      PortAr[2] := RemAddr[5];

      {-- setup the ipstring -----------------------------------------------}
      IpStr := FStr(RemAddr[1]) + '.' +
               FStr(RemAddr[2]) + '.' +
               FStr(RemAddr[3]) + '.' +
               FStr(RemAddr[4]);

      {-- lets connect to the server ---------------------------------------}
      OpenDataConnection := DataThread.ConnectToServer(IpStr, PortNum, ResStr);

      {-- if we succeeded, send an 150 -------------------------------------}
      if Result then
        SendCodeMsg(CurSlot, 150, Format('Connecting to port %d (%s)', [PortNum, IpStr]))
         else SendCodeStr(CurSlot, 425);
    end; { with }
end; { func. OpenDataConnection }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdHelp(const CurSlot: Longint; var ParamString: String);
var HelpCmd: String;

procedure SendHelpMsg(Msg: String);
begin
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    FtpSession.SendStrLn(Format('%d Syntax: %s', [214, Msg]));
  HelpCmd := '';
end; { proc. SendHelpMsg }

begin
  {-- Check if we need to send a list of all supported commands -------------}
  if ParamString = '' then
    with ftpConnectionRec(Connections[Curslot].SrvData^) do
      begin
        FtpSession.SendStrLn('214-The following commands are recognized (* ==> ''s unimplemented).');

        FtpSession.SendStrLn('   ABOR');
        FtpSession.SendStrLn('   ALLO');
        FtpSession.SendStrLn('   APPE');
        FtpSession.SendStrLn('   CDUP');
        FtpSession.SendStrLn('   CWD');
        FtpSession.SendStrLn('   DELE');
        FtpSession.SendStrLn('   HELP');
        FtpSession.SendStrLn('   LIST');
        FtpSession.SendStrLn('   MKD');
        FtpSession.SendStrLn('   MODE');
        FtpSession.SendStrLn('   NLST');
        FtpSession.SendStrLn('   NOOP');
        FtpSession.SendStrLn('   PASS (*)');
        FtpSession.SendStrLn('   PASV (*)');
        FtpSession.SendStrLn('   PORT');
        FtpSession.SendStrLn('   PWD');
        FtpSession.SendStrLn('   QUIT (*)');
        FtpSession.SendStrLn('   REST');
        FtpSession.SendStrLn('   RETR');
        FtpSession.SendStrLn('   RMD');
        FtpSession.SendStrLn('   RNFR');
        FtpSession.SendStrLn('   RNTO');
        FtpSession.SendStrLn('   SITE');
        FtpSession.SendStrLn('   STAT');
        FtpSession.SendStrLn('   STOR (*)');
        FtpSession.SendStrLn('   STRU');
        FtpSession.SendStrLn('   SYST');
        FtpSession.SendStrLn('   TYPE');
        FtpSession.SendStrLn('   USER (*)');

        FtpSession.SendStrLn('214  HELP command successful.');
      end
        else begin      { Send a command depending upon twhat the user wants }
               HelpCmd := SUpCase(Trim(ExtractWord(ParamString, 1, defExtractWord, false, false)));

               if HelpCmd = 'ABOR' then SendHelpMsg('ABOR (abort operation)');
               if HelpCmd = 'ALLO' then SendHelpMsg('ALLO (allocate storage vacuously)');
               if HelpCmd = 'APPE' then SendHelpMsg('APPE <file-name>');
               if HelpCmd = 'CDUP' then SendHelpMsg('CDUP change to parent directory');
               if HelpCmd = 'CWD'  then SendHelpMsg('CWD <directory-name>');
               if HelpCmd = 'DELE' then SendHelpMsg('DELE <file-name>');
               if HelpCmd = 'HELP' then SendHelpMsg('HELP <string>');
               if HelpCmd = 'LIST' then SendHelpMsg('LIST <path-name>');
               if HelpCmd = 'MKD'  then SendHelpMsg('MKD <path-name>');
               if HelpCmd = 'MODE' then SendHelpMsg('MODE (specify transfer mode)');
               if HelpCmd = 'NLST' then SendHelpMsg('NLST <path-name>');
               if HelpCmd = 'NOOP' then SendHelpMsg('NOOP');
               if HelpCmd = 'PASS' then SendHelpMsg('PASS <password>');
               if HelpCmd = 'PASV' then SendHelpMsg('PASV (set server in passive mode)');
               if HelpCmd = 'PORT' then SendHelpMsg('PORT <b0,b1,b2,b3,b4,b5>');
               if HelpCmd = 'PWD'  then SendHelpMsg('PWD (return current directory)');
               if HelpCmd = 'QUIT' then SendHelpMsg('QUIT (terminate service)');
               if HelpCmd = 'REST' then SendHelpMsg('REST (marker)');
               if HelpCmd = 'RETR' then SendHelpMsg('RETR <file-name>');
               if HelpCmd = 'RMD'  then SendHelpMsg('RMD <path-name>');
               if HelpCmd = 'RNFR' then SendHelpMsg('RNFR <file-name>');
               if HelpCmd = 'RNTO' then SendHelpMsg('RNTO <file-name>');
               if HelpCmd = 'SITE' then SendHelpMsg('SITE (site-specific commands)');
               if HelpCmd = 'SMNT' then SendHelpMsg('SMNT <pathname>');
               if HelpCmd = 'STAT' then SendHelpMsg('STAT (get server status)');
               if HelpCmd = 'STOR' then SendHelpMsg('STOR <file-name>');
               if HelpCmd = 'STRU' then SendHelpMsg('STRU (specify file structure)');
               if HelpCmd = 'SYST' then SendHelpMsg('SYST (get operating system type)');
               if HelpCmd = 'TYPE' then SendHelpMsg('TYPE < I >');
               if HelpCmd = 'USER' then SendHelpMsg('USER <username>');

               {-- Return an error if we dont know it -----------------------}
               if HelpCmd <> '' then
                with ftpConnectionRec(Connections[Curslot].SrvData^) do
                 FtpSession.SendStrLn(Format('%d Unknown command %s', [501, HelpCmd]));
            end; { else }
end; { proc. cmdHelp }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdPort(const CurSlot: Longint; var ParamString: String);
begin
  {-- We get the port to connect to as follows: ----------------------------}
  {-- PORT aa,bb,cc,dd,ee,ff -----------------------------------------------}
  {-- aa,bb,cc,dd are the 4 bytes of the IP address, ee and ff are the -----}
  {-- port numbers ---------------------------------------------------------}

  {-- extract the ip-address -----------------------------------------------}
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    begin
      RemAddr[1] := FVal(ExtractWord(ParamString, 1, defExtractWord + [','], false, false));
      RemoveWordNr(ParamString, 1, defExtractword + [','], true);
      RemAddr[2] := FVal(ExtractWord(ParamString, 1, defExtractWord + [','], false, false));
      RemoveWordNr(ParamString, 1, defExtractword + [','], true);
      RemAddr[3] := FVal(ExtractWord(ParamString, 1, defExtractWord + [','], false, false));
      RemovewordNr(ParamString, 1, defExtractword + [','], true);
      RemAddr[4] := FVal(ExtractWord(ParamString, 1, defExtractWord + [','], false, false));
      RemovewordNr(ParamString, 1, defExtractword + [','], true);
   end; { with }

  {-- extract the port number ----------------------------------------------}
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    begin
      RemAddr[5] := FVal(ExtractWord(ParamString, 1, defExtractWord + [','], false, false));
      RemovewordNr(ParamString, 1, defExtractword + [','], true);
      RemAddr[6] := FVal(ExtractWord(ParamString, 1, defExtractWord + [','], false, false));
      RemovewordNr(ParamString, 1, defExtractword + [','], true);
    end; { with }

  {-- now validate the send parameters and respond -------------------------}
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    begin
      IsPassive := FALSE;
      UseDataConn := FALSE;

      if (RemAddr[1] = 0) then
        begin
          SendCodeMsg(CurSlot, 501, 'Parameter syntax error');
        end
          else SendCodeMsg(CurSlot, 200, 'PORT command successful');
    end; { with }
end; { proc. cmdPort }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function ConvDirName(DirName: String): String;
type
  CharSet = set of char;
const IllegalChars : CharSet = [#32, '/', '\'];

var Counter: Longint;
    TmpStr : String;
begin
  TmpStr := '';

  {-- now loop to remove at least the spaces ------------------------------}
  for Counter := 01 to Length(DirName) do
    begin
      if DirName[Counter] in IllegalChars then
        TmpStr := TmpStr + '_'
         else TmpStr := TmpStr + DirName[Counter];
    end; { for }

  ConvDirName := TmpStr;
end; { func. ConvDirName }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function MakeFileEntry(IsFile   : Boolean;
                       SizeStr  : String;
                       DateStr  : String;
                       FileName : String;
                       IsNLST   : Boolean): String;
var AccStr : String;                                        { Access string }
    UnkStr : String;                      { dont really know what it is for }
    UName,                                                       { username }
    GName  : String;                                            { Groupname }
begin
  {-- now create the access list -------------------------------------------}
  if IsFile then
    AccStr := '-r--r--r--'
     else AccStr := 'dr-xr-xr-x';

  {-- unknown string -------------------------------------------------------}
  UnkStr := '1';

  {-- username -------------------------------------------------------------}
  UName := 'user';

  {-- groupname-------------------------------------------------------------}
  GName := 'group';

  {-- and add the "filename" -----------------------------------------------}
  MakeFileEntry := AccStr + #32 +                              { ---------- }
                   MakeLen(UnkStr, 3, Space, true, false) + #32 +     {   1 }
                   MakeLen(UName, 8, Space, false, false) + #32 +    { user }
                   MakeLen(GName, 8, Space, false, false) + #32 +   { group }
                   MakeLen(SizeStr, 10, Space, true,  false)+ #32 +{ (size) }
                   DateStr + #32 +                                   { date }
                   FileName;

  {-- if we want a simple listing, override this ---------------------------}
  if IsNLST then
    MakeFileEntry := FileName;
end; { func. MakeFileEntry }


(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function MakeDirString(DirName: String; IsNLST: Boolean): String;
var DateStr: String;
begin
  {-- Create a datestr ----------------------------------------------------}
  DateStr := FormatDateTime('mmm dd  yyyy', Now);

  {-- and build it all now ------------------------------------------------}
  Result := MakeFileEntry(false,                              { is a file }
                          '1024',                                  { size }
                          DateStr, { directories always have current date }
                          ConvDirName(DirName),
                          IsNLST);

  {-- if we want a simple listing, override this ---------------------------}
  if IsNLST then
    Result := DirName;
end; { func. MakeDirString }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure SendGroupList(const CurSlot: Longint; WCName: String; IsNLST: Boolean);
var Group_F : pFileObj;
    GroupInf: GroupRecord;
begin
  {-- Initialize some variables --------------------------------------------}
  if WcName = '' then
    WcName := '*.*';

  {-- we have a working dataconnection, use it as we want ------------------}
  With ftpConnectionRec(Connections[CurSlot].SrvData^).DataThread do
    begin
      {-- We send a list of the groups, spaces replaced with ---------------}
      {-- underscores ------------------------------------------------------}

      {-- open the group file ----------------------------------------------}
      New(Group_F, Init);
      Group_F^.Assign(FGroupsFileName);
      Group_F^.FileMode := ReadMode + DenyNone;
      Group_F^.Open(1);

      {-- now loop through it ------------------------------------------}
      while NOT Group_F^.EOF do
        begin
          {-- read the grouprecord -------------------------------------}
          Group_F^.BlkRead(GroupInf, SizeOf(GroupRecord));

          {-- convert the groupname to a directory-listing -------------}
          if CheckGroupAccess(GroupInf, ftpConnectionRec(Connections[CurSlot].SrvData^).Exitinfo) then
           if IsWildCard(WcName, GroupInf.Name) then
             SendStrLn(MakeDirString(GroupInf.Name, IsNLST));
        end; { while }

      {-- and close the file -------------------------------------------}
      Dispose(Group_F, Done);
    end; { with }
end; { proc. SendGroupList }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure SendAreaList(const CurSlot: Longint; GroupNum: Longint; WCName: String;
                       IsNlst: Boolean);
var Area_F  : pFileObj;
    FilesInf: FilesRecord;
begin
  {-- Initialize some variables --------------------------------------------}
  if WcName = '' then
    WcName := '*.*';

  {-- we have a working dataconnection, use it as we want ------------------}
  With ftpConnectionRec(Connections[CurSlot].SrvData^).DataThread do
    begin
      {-- We send a list of the groups, spaces replaced with ---------------}
      {-- underscores ------------------------------------------------------}

      {-- open the group file ----------------------------------------------}
      New(Area_F, Init);
      Area_F^.Assign(FilesFileName);
      Area_F^.FileMode := ReadMode + DenyNone;
      Area_F^.Open(1);

      {-- now loop through it ------------------------------------------}
      while NOT Area_F^.EOF do
        begin
          {-- read the filesrecord -------------------------------------}
          Area_F^.BlkRead(FilesInf, SizeOf(FilesRecord));

          {-- if we have access to this group --------------------------}
          if CheckFileAreaAccess(FilesInf,
                                 FALSE,                      { download }
                                 TRUE,                     { groupcheck }
                                 FALSE,                  { upload check }
                                 TRUE,                    { list access }
                                 GroupNum,
                                 ftpConnectionRec(
                                   Connections[CurSlot].SrvData^).Exitinfo){ exitinfo } then
            begin
              {-- convert the groupname to a directory-listing -------------}
              if IsWildCard(WcName, FilesInf.Name) then
                SendStrLn(MakeDirString(FilesInf.Name, IsNLST));
            end; { if }

        end; { while }

      {-- and close the file -----------------------------------------------}
      Dispose(Area_F, Done);
    end; { with }
end; { proc. SendAreaList }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function Dos2FtpDate(Date: Longint): String;
begin
  {-- Create a datestr ----------------------------------------------------}
  Result := FormatDateTime('mmm dd  yyyy', FileDateToDateTime(Date));
end; { func. Dos2FtpDate }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure SendFileList(const CurSlot: Longint; CurArea: Longint; WCName: String;
                       IsNlst: Boolean);
var AreaObj     : FdbFileObj;
    FilesInf    : FilesRecord;
    TotalItems  : Longint;
    ItemCounter : Longint;
    TmpStr      : String;
begin
  {-- Initialize some variables --------------------------------------------}
  if WcName = '' then
    WcName := '*.*';

  {-- we have a working dataconnection, use it as we want ------------------}
  With ftpConnectionRec(Connections[CurSlot].SrvData^).DataThread do
    begin
      {-- We send a list of the groups, spaces replaced with ---------------}
      {-- underscores ------------------------------------------------------}

      {-- open the group file ----------------------------------------------}
      AreaObj.Init(CurArea, false);                        { create = false }
      if AreaObj.GetError <> 0 then
        begin
          AreaObj.Done;
          EXIT;
        end; { if }

      {-- get the total number of items ------------------------------------}
      TotalItems := AreaObj.TotalRecords;
      ItemCounter := 0;

      {-- loop through all files -------------------------------------------}
      while (ItemCounter < TotalItems) AND (TotalItems > 0) do
        begin
          AreaObj.Read(ItemCounter);             { Read header sequentially }
          Inc(ItemCounter);

          {-- if the file is missing, we dont show it ----------------------}
          if (NOT AreaObj.IsMissing) then
           if (NOT AreaObj.IsComment) then
            if (AreaObj.GetPassword = '') then
             if (NOT AreaObj.IsDeleted) then
              if (NOT AreaObj.IsNotAvail) then
             begin
               {-- create the filestring -----------------------------------}
               TmpStr := MakeFileEntry(true,                       { isFile }
                                       FStr(AreaObj.GetSize),      { FileSz }
                                       Dos2FtpDate(AreaObj.GetFileDate),
                                       AreaObj.GetFileName,
                                       IsNLST);

               {-- and sed the file ----------------------------------------}
               if IsWildCard(WcName, AreaObj.GetFileName) then
                 SendStrLn(TmpStr);
             end; { if }
        end; { while }

      AreaObj.Done;
    end; { with }
end; { proc. SendFileList }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function StripIllegal(DirName: String): String;
var Counter: Longint;
begin
  {-- first change all /'s to \'s ------------------------------------------}
  for Counter := 01 to Length(DirName) do
    if DirName[Counter] = '/' then Dirname[Counter] := '\';

  {-- First remove all ../'s and ./'s --------------------------------------}
  {-- (we do not support it at all -----------------------------------------}
  while Pos('..\', DirName) > 0 do
    Replace('..\', '', DirName);

  while Pos('.\', DirName) > 0 do
    Replace('.\', '', DirName);

  {-- and set the contents -------------------------------------------------}
  StripIllegal := DirName;
end; { func. StripIllegal }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure BreakUpDirStr(DirName: String;
                        var GroupName, AreaName, FileName, WildCardStr: String;
                        var StartAtRoot: Boolean;
                        var ListArea, ListGroup: Longint);
var Counter  : Longint;
    WordOne  : Array[1..3] of String;
begin
  {-- Initialize the variables ---------------------------------------------}
  StartAtRoot := FALSE;
  WordOne[1] := '';
  WordOne[2] := '';
  WordOne[3] := '';
  Groupname := '';
  AreaName := '';
  FileName := '';
  WildCardStr := '';

  {-- first change all \'s to /'s ------------------------------------------}
  for Counter := 01 to Length(DirName) do
    if DirName[Counter] = '\' then Dirname[Counter] := '/';

  {-- First remove all ../'s and ./'s --------------------------------------}
  {-- (we do not support it at all -----------------------------------------}
  while Pos('../', DirName) > 0 do
    Replace('../', '', DirName);

  while Pos('./', DirName) > 0 do
    Replace('./', '', DirName);

  {-- do we start at the root? ---------------------------------------------}
  if DirName[1] = '/' then
    StartAtRoot := TRUE;

  {-- now lets see how many path seperators there are ----------------------}
  WordOne[1] := ExtractWord(DirName, 1, ['/'], false, false);
  RemoveWordNr(DirName, 1, ['/'], false);

  if Length(DirName) > 0 then
    begin
      WordOne[2] := ExtractWord(DirName, 1, ['/'], false, false);
      RemoveWordNr(DirName, 1, ['/'], false);
    end; { if }

  if Length(DirName) > 0 then
    begin
      WordOne[3] := ExtractWord(DirName, 1, ['/'], false, false);
      RemoveWordNr(DirName, 1, ['/'], false);
    end; { if }

  {-- move the wildcard to its own place -----------------------------------}
  if (Pos('*', WordOne[3]) > 0) OR
      (Pos('?', WordOne[3]) > 0) then
    begin
      WildCardStr := WordOne[3];
      WordOne[3] := '';
    end; { if }

  if (Pos('*', WordOne[2]) > 0) OR
      (Pos('?', WordOne[2]) > 0) then
    begin
      WildCardStr := WordOne[2];
      WordOne[2] := '';
    end; { if }

  if (Pos('*', WordOne[1]) > 0) OR
      (Pos('?', WordOne[1]) > 0) then
    begin
      WildCardStr := WordOne[1];
      WordOne[1] := '';
    end; { if }

  {-- reset everything if we start from the root ---------------------------}
  if StartAtRoot then
    begin
      ListArea := -1;
      ListGroup := -1;
    end; { if }

  {-- now put the result in the correct boxes ------------------------------}
  if WordOne[3] <> '' then
    begin
      Groupname := WordOne[1];
      AreaName := WordOne[2];
      FileName := WordOne[3];
    end { if }
      else begin
             if WordOne[2] <> '' then
               begin
                 if ListGroup > 0 then
                   begin
                     AreaName := WordOne[1];
                     FileName := WordOne[2];
                   end
                     else begin
                            GroupName := WordOne[1];
                            AreaName := WordOne[2];
                          end; { else }
               end
                 else begin
                        if (ListGroup > 0) AND (ListArea > 0) then
                          FileName := WordOne[1];

                        if (ListGroup > 0) AND (ListArea < 0) then
                          AreaName := WordOne[1];

                        if (ListGroup < 0) then
                          GroupName := WordOne[1];
                      end; { else }
           end; { else }
end; { proc. BreakUpDirStr }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function SearchGroupName(CurSlot:Longint; GroupName: String): Longint;
var Group_F : pFileObj;
    GroupInf: GroupRecord;
begin
  {-- Initialize some variables -------------------------------------------}
  SearchGroupName := -1;
  GroupName := Trim(SUpCase(GroupName));

  {-- open the group file -------------------------------------------------}
  New(Group_F, Init);
  Group_F^.Assign(FGroupsFileName);
  Group_F^.FileMode := ReadMode + DenyNone;
  Group_F^.Open(1);

  {-- now loop through it -------------------------------------------------}
  while NOT Group_F^.EOF do
    begin
      {-- read the grouprecord --------------------------------------------}
      Group_F^.BlkRead(GroupInf, SizeOf(GroupRecord));

      {-- convert the groupname to a directory-listing --------------------}
      if CheckGroupAccess(GroupInf, ftpConnectionRec(Connections[CurSlot].SrvData^).Exitinfo) then
        begin
          if GroupName = SUpcase(ConvDirName(GroupInf.Name)) then
            begin
              SearchGroupName := GroupInf.AreaNum;
            end; { if }
        end; { if }
    end; { while }

  {-- and close the file --------------------------------------------------}
  Dispose(Group_F, Done);
end; { func. SearchGroupName }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function SearchAreaName(CurSlot:Longint; AreaName: String): Longint;
var Area_F  : pFileObj;
    FilesInf: FilesRecord;
begin
  {-- Initialize some variables -------------------------------------------}
  SearchAreaName := -1;
  AreaName := Trim(SUpCase(AreaName));

  {-- we have a working dataconnection, use it as we want ------------------}
  With ftpConnectionRec(Connections[CurSlot].SrvData^).DataThread do
    begin
      {-- We send a list of the areas, spaces replaced with underscores ----}

      {-- open the group file ----------------------------------------------}
      New(Area_F, Init);
      Area_F^.Assign(FilesFileName);
      Area_F^.FileMode := ReadMode + DenyNone;
      Area_F^.Open(1);

      {-- now loop through it ------------------------------------------}
      while NOT Area_F^.EOF do
        begin
          {-- read the filesrecord -------------------------------------}
          Area_F^.BlkRead(FilesInf, SizeOf(FilesRecord));

          {-- if we have access to this group --------------------------}
          if CheckFileAreaAccess(FilesInf,
                                 FALSE,                      { download }
                                 FALSE,                    { groupcheck }
                                 FALSE,                  { upload check }
                                 TRUE,                    { list access }
                                 0,
                                 ftpConnectionRec(
                                  Connections[CurSlot].SrvData^).Exitinfo){ exitinfo } then
            begin
              if AreaName = SUpcase(ConvDirName(FilesInf.Name)) then
                begin
                  SearchAreaName := FilesInf.AreaNum;
                  BREAK;
                end; { if }
            end; { if }

        end; { while }

      {-- and close the file -----------------------------------------------}
      Dispose(Area_F, Done);
    end; { with }
end; { func. SearchAreaName }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure GetPartsFromDir(var   ParamString: String;
                          var   WildCardStr: String;
                          const CurSlot    : Longint;
                          var   InvalidDir : Boolean;
                          var   ListGroup,
                                ListArea   : Longint);
var DirStr      : String;

    GroupName,
    AreaName,
    FileName    : String;

    StartRoot   : Boolean;

begin
  {-- Initialize some variables --------------------------------------------}
  GroupName := '';
  AreaName := '';
  FileName := '';
  WildCardStr := '';
  InvalidDir := FALSE;
  StartRoot := FALSE;

  {-- Dirstring ----------------------------------------------------}
  DirStr := ExtractWord(ParamString, 1, defExtractWord, true, false);

  {-- BreakUpDirStr ------------------------------------------------}
  if DirStr <> '' then
    begin
      {-- break it up in area and stuff ----------------------------}
      BreakUpDirStr(DirStr,
                    GroupName,
                    AreaName,
                    FileName,
                    WildCardStr,
                    StartRoot,
                    ListArea,
                    ListGroup);

      {-- now make a judgement what to list -------------------------}
      if StartRoot then
        begin
          ListGroup := -1;
          ListArea := -1;
        end; { if }

      if GroupName <> '' then
        begin
          ListGroup := SearchGroupName(CurSlot, GroupName);
          if ListGroup < 0 then
            InvalidDir := TRUE;
        end; { if }

      if AreaName <> '' then
        begin
          ListArea := SearchAreaName(CurSlot, AreaName);
          if ListArea < 0 then
            InvalidDir := TRUE;
        end; { if }

      if WildCardStr = '' then
        WildCardStr := FileName;
    end; { if }
end; { proc. GetPartsFromdir }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure SendPleaseAuthFirst(const CurSlot: Longint);
begin
  SendCodeMsg(CurSlot, 530, 'Please login with USER and PASS first');
end; { proc. SendPleaseAuthFirst }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdSendList(const CurSlot: Longint; var ParamString: String; NLST: Boolean);
var ResStr      : String;
    DirStr      : String;

    InvalidDir  : Boolean;
    WildCardStr : String;
    ListArea    : Longint;
    ListGroup   : Longint;
begin
  {-- Initialize some variables --------------------------------------------}
  WildCardStr := '';

  {-- we default to the current group/area listing -------------------------}
  ListArea := ftpConnectionRec(Connections[CurSlot].SrvData^).CurArea;
  ListGroup := ftpConnectionRec(Connections[CurSlot].SrvData^).CurGroup;

  {-- open data ------------------------------------------------------------}
  if OpenDataConnection(CurSlot, ResStr) then
    begin
      {-- we have a working dataconnection, use it as we want --------------}
      With ftpConnectionRec(Connections[CurSlot].SrvData^).DataThread do
        begin
          {-- if we are at the root, show a group list ---------------------}
          with ftpConnectionRec(Connections[Curslot].SrvData^) do
            begin
              {-- Break it up ----------------------------------------------}
              GetPartsFromDir(ParamString,
                              WildCardStr,
                              CurSlot,
                              InvalidDir,
                              ListGroup,
                              ListArea);

              {-- send the grouplist ---------------------------------------}
              if (ListArea < 0) AND (ListGroup < 0) then
                begin
                  SendGroupList(CurSlot, WildCardStr, NLST);
                end
                  else begin
                         {-- send the arealist -----------------------------}
                         if (ListArea < 0) then
                           SendAreaList(CurSlot, ListGroup, WildCardStr, NLST);

                         {-- send the filelist -----------------------------}
                         if (ListArea > 0) AND (ListGroup > 0) then
                           begin
                             SendFileList(CurSlot,
                                          ListArea,
                                          WildCardStr,
                                          NLST);
                           end; { if }
                       end; { if }

            end; { with }
        end; { with }

      {-- and eventually close it again ------------------------------------}
      CloseDataConnection(CurSlot, true);
    end { if }
      else begin
             {-- send an error ---------------------------------------------}
             SendCodeMsg(CurSlot, 425, 'Unable to open data connection');
           end; { if }
end; { proc. cmdSendList }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdAllo(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeMsg(CurSlot, 200, 'Go ahead');
end; { proc. cmdAllo }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdMkd(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeStr(CurSlot, 502);
end; { proc. cmdMkd }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdRmd(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeStr(CurSlot, 502);
end; { proc. cmdRmd }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdRest(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeStr(CurSlot, 502);
end; { proc. cmdRest }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdRnFr(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeStr(CurSlot, 502);
end; { proc. cmdRnFr }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdRnTo(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeStr(CurSlot, 502);
end; { proc. cmdRnTo }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdRetr(const CurSlot: Longint; var ParamString: String);
var ResStr      : String;
    WildCardStr : String;
    ListGroup   : Longint;
    ListArea    : Longint;

    FilesInf    : FilesRecord;

    Temp_F      : pFileObj;
    InvalidDir  : Boolean;
    DidRead     : Longint;
    Counter     : Longint;
    DidSent     : Longint;
    TmpBuf      : Array[0..4095] of Char;
begin
  {-- we default to the current group/area listing -------------------------}
  ListArea := ftpConnectionRec(Connections[CurSlot].SrvData^).CurArea;
  ListGroup := ftpConnectionRec(Connections[CurSlot].SrvData^).CurGroup;

  {-- make sure we can retrieve the file etc, then actually send it --------}
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    begin
      {-- Break it up ------------------------------------------------------}
      GetPartsFromDir(ParamString,
                      WildCardStr,
                      CurSlot,
                      InvalidDir,
                      ListGroup,
                      ListArea);

      {-- Get the file path ------------------------------------------------}
      GetFilesRecord(FilesInf, ListArea, true);

      {-- if we have access to this group ----------------------------------}
      if NOT CheckFileAreaAccess(FilesInf,
                                 TRUE,                       { download }
                                 FALSE,                    { groupcheck }
                                 FALSE,                  { upload check }
                                 FALSE,                   { list access }
                                 ListGroup,
                                 ftpConnectionRec(
                                   Connections[CurSlot].SrvData^).Exitinfo){ exitinfo } then
       WildCardStr := '';

      {-- we should add checks wether the file is unavailable, missing, etc-}
      {-- to it, but currently we only do that at the listing part ---------}

      {-- make sure we have download access --------------------------------}
      if WildcardStr <> '' then
        begin
          {-- Now try opening the file -------------------------------------}
          New(Temp_F, Init);
          Temp_F^.Assign(FilesInf.FilePath + StripIllegal(WildCardStr));
          Temp_F^.FileMode := ReadMode + DenyNone;

          if Temp_F^.Open(1) then
            begin
              {-- open data connections ------------------------------------}
              if OpenDataConnection(CurSlot, ResStr) then
                begin
                  {-- we have a working dataconnection, use it as we want --}
                  while (NOT Temp_F^.EOF) AND (DataThread.ConnectionAlive) do
                    begin
                      {-- read the required data ---------------------------}
                      DidRead := Temp_F^.BlkRead(TmpBuf, SizeOf(TmpBuf));

                      {-- create a small loop to ensure we send everything -}
                      Counter := 0;

                      while (Counter < DidRead) AND (DataThread.ConnectionAlive) do
                        begin
                          DataThread.SendBuf(TmpBuf[Counter], (DidRead - Counter), DidSent);

                          if DidSent > 0 then
                            Inc(Counter, DidSent)
                              else DataThread.DoSleep(100);
                        end; { while }

                    end; { while }

                  {-- and eventually close it again ------------------------}
                  CloseDataConnection(CurSlot, true);
                end; { if }
            end { file open }
              else SendCodeStr(CurSlot, 450);

          {-- and close the file -------------------------------------------}
          Dispose(Temp_F, Done);
        end { if }
         else begin
                SendCodeStr(CurSlot, 450);
              end; { file is not available (security) }
    end; { with }
end; { proc. cmdRetr }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function GetPwdStr(CurSlot: Longint): String;
var CurDir    : String;
    GroupInf  : GroupRecord;
    FilesInf  : FilesRecord;
begin
  {-- default to root location ---------------------------------------------}
  CurDir := '/';

  if ftpConnectionRec(Connections[CurSlot].SrvData^).CurGroup > 0 then
    begin
      {-- now get the current directory for the group ----------------------}
      GetGroupRecord(ftpConnectionRec(Connections[CurSlot].SrvData^).CurGroup, true, false, GroupInf);
      CurDir := CurDir + ConvDirName(GroupInf.Name) + '/';

      {-- and if we are in an area, add that too ---------------------------}
      if ftpConnectionRec(Connections[CurSlot].SrvData^).CurArea > 0 then
        begin
          GetFilesRecord(FilesInf, ftpConnectionRec(
              Connections[CurSlot].SrvData^).CurArea, true);

          CurDir := CurDir + ConvDirName(FilesInf.Name) + '/';
        end; { if }
    end; { if }

  {-- and return the string ------------------------------------------------}
  GetPwdStr := CurDir;
end; { func. GetPwdStr }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdPwd(const CurSlot: Longint; var ParamString: String);
begin
  {-- and send the result --------------------------------------------------}
  SendCodeMsg(CurSlot, 257, Format('"%s" is your current location', [GetPwdStr(CurSlot)]));
end; { proc. cmdPwd }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdMode(const CurSlot: Longint; var ParamString: String);
begin
  if SupCase(Trim(ParamString)) = 'S' then
    begin
      SendCodeMsg(CurSlot, 200, 'mode S ok');
    end
      else SendCodeMsg(CurSlot, 504, 'Please use S(tream) mode');
end; { proc. CmdMode }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdType(const CurSlot: Longint; var ParamString: String);
begin
  if (SupCase(Trim(ParamString)) = 'I') OR (SupCase(Trim(ParamString)) = 'A') then
    begin
      SendCodeMsg(CurSlot, 200, 'mode I (binary) ok');
    end
      else SendCodeMsg(CurSlot, 504, 'Please use I (binary) mode');
end; { proc. CmdMode }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdStru(const CurSlot: Longint; var ParamString: String);
begin
  if SupCase(Trim(ParamString)) = 'F' then
    begin
      SendCodeMsg(CurSlot, 200, 'F OK');
    end
      else SendCodeMsg(CurSlot, 504, 'Please use F (file) mode');
end; { proc. CmdStru }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdSmnt(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeStr(CurSlot, 502);
end; { proc. cmdSmnt }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdappe(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeStr(CurSlot, 502);
end; { proc. cmdAppe }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdOpts(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeStr(CurSlot, 502);
end; { proc. cmdOpts }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdNoop(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeMsg(CurSlot, 200, '...');
end; { proc. cmdNoop }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdUser(const CurSlot: Longint; var ParamString: String);
var TmpUser  : Longint;
begin
  {-- for now, we dont allow anonymous FTP access --------------------------}
  TmpUser := SearchUser(ParamString);
  if TmpUser < 0 then
    begin
      ftpConnectionRec(Connections[CurSlot].SrvData^).UserName := '';
      ftpConnectionRec(Connections[CurSlot].SrvData^).PasswordCRC := -1;
      ftpConnectionRec(Connections[CurSlot].SrvData^).Validated := false;
      ftpConnectionRec(Connections[CurSlot].SrvData^).UserRec := TmpUser;
    end
      else begin
             ftpConnectionRec(Connections[CurSlot].SrvData^).UserName := ParamString;
             ftpConnectionRec(Connections[CurSlot].SrvData^).UserRec := TmpUser;
             ftpConnectionRec(Connections[CurSlot].SrvData^).Validated := false;
           end; { else }

  {-- we always ask for a password to not give away our userlist ;) --------}
  SendCodeMsg(CurSlot, 331, 'User name, okay. Please send password.');
end; { proc. cmdUser }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdPass(const CurSlot: Longint; var ParamString: String);
var UserExt  : userExtensionRecord;
begin
  {-- for now, we dont allow anonymous FTP access --------------------------}

  {-- if a username was entered, try vlaidating the password ---------------}
  if ftpConnectionRec(Connections[CurSlot].SrvData^).UserRec >= 0 then
    begin
      {-- calculcate the given CRC -----------------------------------------}
      ftpConnectionRec(Connections[CurSlot].SrvData^).PasswordCRC := RaCrc(ParamString, true);

      {-- retrieve the users password --------------------------------------}
      GetUserRecord(SearchUser(ftpConnectionRec(Connections[CurSlot].SrvData^).UserName),
                    ftpConnectionRec(Connections[CurSlot].SrvData^).Exitinfo.UserInfo,
                    UserExt,
                    false);

      {-- and validate with the given password -----------------------------}
      if ftpConnectionRec(Connections[CurSlot].SrvData^).PasswordCRC <>
          ftpConnectionRec(Connections[CurSlot].SrvData^).Exitinfo.UserInfo.PasswordCRC then
            begin
              ftpConnectionRec(Connections[CurSlot].SrvData^).UserName := '';
              ftpConnectionRec(Connections[CurSlot].SrvData^).UserRec := 0;
              ftpConnectionRec(Connections[CurSlot].SrvData^).Validated := false;
            end { if }
              else ftpConnectionRec(Connections[CurSlot].SrvData^).Validated := true;

      {-- and the user has logged on ---------------------------------------}
      if ftpConnectionRec(Connections[CurSlot].SrvData^).Validated then
        SendCodeMsg(CurSlot, 230, 'User logged in, proceed.');
    end; { if }

  {-- if the username was not sent, or was invalid deny immedaitly ---------}
  if NOT ftpConnectionRec(Connections[CurSlot].SrvData^).Validated then
    begin
      SendCodeMsg(CurSlot, 530, 'User cannot log in');
    end; { if }
end; { proc. cmdPass }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdSyst(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeMsg(CurSlot, 215, 'UNIX');
end; { proc. cmdSyst }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdDele(const CurSlot: Longint; var ParamString: String);
begin
  SendcodeStr(CurSlot, 502);
end; { proc. cmdDele }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdAbor(const CurSlot: Longint; var ParamString: String);
begin
  SendcodeStr(CurSlot, 502);
end; { proc. cmdabor }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdSite(const CurSlot: Longint; var ParamString: String);
begin
  SendCodeMsg(CurSlot, 500, 'No SITE commands implemented here');
end; { proc. cmdSite }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdPasv(const CurSlot: Longint; var ParamString: String);
begin
  SendcodeStr(CurSlot, 502);
end; { proc. cmdPasv }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdStat(const CurSlot: Longint; var ParamString: String);
begin
  if Trim(ParamString) = '' then
    begin
      SendCodeMsg(CurSlot, 211, Format(ServerName, [GlobalCfg^.RaConfig^.SystemName]) + ' ready.');
    end
      else begin
             {-- send the directory listing over the wire ... --------------}
             ftpConnectionRec(Connections[CurSlot].SrvData^).UseDataConn := TRUE;

             {-- send the filelisting --------------------------------------}
             cmdSendList(CurSlot, ParamString, FALSE);

             {-- and revert to normal operation ----------------------------}
             ftpConnectionRec(Connections[CurSlot].SrvData^).UseDataConn := FALSE;
           end; { else }
end; { proc. cmdStat }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdCdUp(const CurSlot: Longint; var ParamString: String);
begin
  {-- go one directory level up --------------------------------------------}
  if ftpConnectionRec(Connections[CurSlot].SrvData^).CurArea > 0 then
    begin
      ftpConnectionRec(Connections[CurSlot].SrvData^).CurArea := -1;
    end { if }
      else ftpConnectionRec(Connections[CurSlot].SrvData^).CurGroup := -1;

  {-- return the current directory -----------------------------------------}
  cmdPwd(CurSlot, ParamString);
end; { proc. cmdCdUp }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdCwd(const CurSlot: Longint; var ParamString: String);
var WildCardStr : String;

    ListArea,
    ListGroup   : Longint;
    InvalidDir  : Boolean;
begin
  {-- Initialize some variables --------------------------------------------}
  ListArea := ftpConnectionRec(Connections[CurSlot].SrvData^).CurArea;
  ListGroup := ftpConnectionRec(Connections[CurSlot].SrvData^).CurGroup;

  {-- Break the string up in sevearl parts ---------------------------------}
  GetPartsFromDir(ParamString,
                  WildCardStr,
                  CurSlot,
                  InvalidDir,
                  ListGroup,
                  ListArea);

  {-- now set the current directory ----------------------------------------}
  if NOT InvalidDir then
    begin
      ftpConnectionRec(Connections[CurSlot].SrvData^).CurArea := ListArea;
      ftpConnectionRec(Connections[CurSlot].SrvData^).CurGroup := ListGroup;

      {-- and show the directory we changed to -----------------------------}
      SendCodeMsg(CurSlot, 250, Format('Changed directory to "%s"', [GetPwdStr(CurSlot)]));
    end
      else SendCodeStr(CurSlot, 550);
end; { proc. cmdCwd }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure cmdQuit(const CurSlot: Longint; var ParamString: String;
                  var AbortSession: Boolean);
begin
  {-- goodbye --------------------------------------------------------------}
  SendCodeStr(CurSlot, 221);

  {-- andw e allow this to close -------------------------------------------}
  AbortSession := TRUE;
end; { proc. cmdQuit }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function ftp_ExecuteConnection(P: pointer): Longint;
var CurSlot      : Longint;          { Current selected slot of information }
    AbortSession : Boolean;          { Indicates wether to stop his session }

    ATempStr     : AnsiString;                   { String used for commands }
    TempStr      : String;    { Convert it to a "short" string for handling }
    CommandStr   : String;                                 { Actual command }
    ParamString  : String;                       { String of all parameters }
    FtpCommand   : FtpCommandSet;                      { Command in an enum }

    StartTime    : EventTimer;  { Make sure this session wont idle too long }
begin
  {-- Initialize some variables ---------------------------------------------}
  CurSlot := Longint(p);
  Inc(CurrentSessions);
  Inc(TotalSessions);

  {-- Make sure this session wont abort immediatly --------------------------}
  AbortSession := false;

  {-- Create seperate FTP-client for this user ------------------------------}
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    begin
      FtpSession := TTcpServer.Create;
      FtpSession.SockHandle := Connections[CurSlot].ClientRC;
    end; { with }

  {-- Start a timing thread to avoid idling too long ------------------------}
  NewTimer(StartTime, Secs2Tics(SecsTimeOut));

  {-- Send an "HELLO" to the user -------------------------------------------}
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    SendCodeStr(CurSlot, 220);

  {-- Start the loop that receives commands an processes them ---------------}
  repeat
    if ftpConnectionRec(Connections[CurSlot].SrvData^).FtpSession.RecvStrLn(ATempStr, false) then
     with ftpConnectionRec(Connections[Curslot].SrvData^) do
       begin
         TempStr := ATempStr;

         {-- If an string is entered, process it ------------------------------}
         if TempStr <> '' then
          begin
            {-- Reset the timer -----------------------------------------------}
            NewTimer(StartTime, Secs2Tics(SecsTimeOut));

            {-- Write the debugging info --------------------------------------}
            {$IFDEF WITH_DEBUG}
              DebugObj.DebugLog(logString, 'Command: '+TempStr);
            {$ENDIF}

            {-- Extract the exact command and default to "error" --------------}
            CommandStr := SUpCase(FirstWord(TempStr, defExtractword, false));
            ftpCommand := ftpError;

            {-- Extract the parameters ----------------------------------------}
            ParamString := Trim(TempStr);

            {-- Translate the string to a set code ----------------------------}
            Case CommandStr[1] of
              'A' : if CommandStr = 'ALLO' then FtpCommand := ftpAllo
                      else if CommandStr = 'APPE' then FtpCommand := ftpAppe
                       else if CommandStr = 'ABORT' then FtpCommand := ftpAbor;
              'H' : if CommandStr = 'HELP' then FtpCommand := ftpHelp;
              'O' : if CommandStr = 'OPTS' then FtpCommand := ftpOpts;
              'Q' : if CommandStr = 'QUIT' then FtpCommand := ftpQuit;
              'D' : if CommandStr = 'DELE' then FtpCommand := ftpDele;
              'P' : if CommandStr = 'PORT' then FtpCommand := ftpPort
                      else if CommandStr = 'PWD' then FtpCommand := ftppwd
                       else if CommandStr = 'PASV' then FtpCommand := ftpPasv
                        else if CommandStr = 'PASS' then FtpCommand := ftpPass;
              'L' : if CommandStr = 'LIST' then FtpCommand := ftpList;
              'C' : if CommandStr = 'CDUP' then FtpCommand := ftpCdUp
                     else if CommandStr = 'CWD' then FtpCommand := ftpCwd;
              'S' : if CommandStr = 'SMNT' then FtpCommand := ftpSmnt
                     else if CommandStr = 'STRU' then FtpCommand := ftpStru
                      else if CommandStr = 'SYST' then FtpCommand := ftpSyst
                       else if CommandStr = 'SITE' then FtpCommand := ftpSite
                        else if CommandStr = 'STAT' then ftpCommand := ftpStat;
              'T' : if CommandStr = 'TYPE' then FtpCommand := ftpType;
              'R' : if CommandStr = 'RETR' then FtpCommand := ftpRetr
                     else if CommandStr = 'RNFR' then FtpCommand := ftpRnFr
                       else if CommandStr = 'RNTO' then FtpCommand := ftpRnTo
                        else if CommandStr = 'RMD' then FtpCommand := ftpRmd
                         else if CommandStr = 'REST' then FtpCommand := ftpRest;
              'U' : if CommandStr = 'USER' then FtpCommand := ftpUser;
              'N' : if CommandStr = 'NLST' then FtpCommand := ftpNlst
                      else if CommandStr = 'NOOP' then FtpCommand := ftpNoop;
              'M' : if CommandStr = 'MKD' then FtpCommand := ftpMkd
                      else If CommandStr = 'MODE' then FtpCommand := ftpMode;
            end; { case }

            {------------------- Execute the command ----------------------------}
            if (ftpConnectionRec(Connections[CurSlot].SrvData^).Validated) OR
                (FtpCommand = ftpUser) OR (FtpCommand = ftpPass) then
                 begin
                   Case FtpCommand of
                     ftpAllo     : cmdAllo(CurSlot, ParamString);
                     ftpAbor     : cmdAbor(CurSlot, ParamString);
                     ftpHelp     : cmdHelp(CurSlot, ParamString);
                     ftpQuit     : cmdQuit(CurSlot, ParamString, AbortSession);
                     ftpList     : cmdSendList(CurSlot, ParamString, FALSE);
                     ftpMkd      : cmdMkd(CurSlot, ParamString);
                     ftpRmd      : cmdRmd(CurSlot, ParamString);
                     ftpAppe     : cmdAppe(CurSlot, ParamString);
                     ftpRetr     : cmdRetr(CurSlot, ParamString);
                     ftpPort     : cmdPort(CurSlot, ParamString);
                     ftpSite     : cmdSite(CurSlot, ParamString);
                     ftpNlst     : cmdSendList(CurSlot, ParamString, TRUE);
                     ftpRest     : cmdRest(CurSlot, ParamString);
                     ftpSmnt     : cmdSmnt(CurSlot, ParamString);
                     ftpPass     : cmdPass(CurSlot, ParamString);
                     ftpPasv     : cmdPasv(CurSlot, ParamString);
                     ftpSyst     : cmdSyst(CurSlot, ParamString);
                     ftpOpts     : cmdOpts(CurSlot, ParamString);
                     ftpStru     : CmdStru(CurSlot, ParamString);
                     ftpRnFr     : CmdRnFr(CurSlot, ParamString);
                     ftpRnTo     : CmdRnFr(CurSlot, ParamString);
                     ftpNoop     : cmdNoop(CurSlot, ParamString);
                     ftpType     : cmdType(CurSlot, ParamString);
                     ftpStat     : cmdStat(CurSlot, ParamString);
                     ftpDele     : cmdDele(CurSlot, ParamString);
                     ftpUser     : cmdUser(CurSlot, ParamString);
                     ftpMode     : cmdMode(CurSlot, ParamString);
                     ftpCdup     : cmdCdUp(CurSlot, ParamString);
                     ftpCwd      : cmdCwd(CurSlot, ParamString);
                     ftpPwd      : cmdPwd(CurSlot, ParamString);
                     ftpError    : begin
                                     RaLog('>', Format('[FTPSRV] [%s] Bad formed command (%s)', [Connections[CurSlot].IpAddr, TempStr]));
                                     SendCodeStr(CurSlot, 500);
                                   end; { error }
                   end { case }
                 end { if authenticated }
                   else begin
                          {-- user is not authenticated - let him auth first-}
                          SendPleaseAuthFirst(CurSlot);
                        end; { if }
          end { if }
       end { with }
        else ftpConnectionRec(Connections[CurSlot].SrvData^).FtpSession.DoSleep(1);

  until (AbortSession) OR (NOT ftpConnectionRec(Connections[CurSlot].SrvData^).FtpSession.ConnectionAlive)
         OR (TimerExpired(StartTime));

  {-- If this session idle timed out, report this ---------------------------}
  if (TimerExpired(StartTime)) then
    begin
      RaLog('>', Format('[FTPSRV] [%s] Connection timedout', [Connections[CurSlot].IpAddr]));
    end; { if }

  {-- Close down the connection ---------------------------------------------}
  with ftpConnectionRec(Connections[Curslot].SrvData^) do
    begin
      FtpSession.Disconnect;
      FtpSession.Free;
    end; { with }

  {-- Let the log know what we are doing ------------------------------------}
  RaLog('>', Format('[FTPSRV] [%s] Connection closed', [Connections[CurSlot].IpAddr]));

  {-- Clear the variables ---------------------------------------------------}
  Connections[CurSlot].IpAddr := '';
  Connections[CurSlot].StartTimeStr := '';
  Connections[CurSlot].ClientRC := -1;
  ftpConnectionRec(Connections[CurSlot].SrvData^).UserName := '';
  ftpConnectionRec(Connections[CurSlot].SrvData^).PasswordCRC := -1;
  FreeMem(Connections[CurSlot].SrvData);
  Connections[CurSlot].srvData := nil;
  Connections[CurSlot].Name := '';

  Dec(CurrentSessions);

  ExitThisThread;
end; { func. FtpServerThread }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

{$S-}
function ftp_ServerThread(P: pointer): Longint;
var ClientAddr  : PSockAddr;                     { Address socket structure }
    ClientRC    : Longint;                   { Return code for socket calls }
    TempLen     : Longint;                { Variable used for Accept() call }

    Counter     : Longint;                                   { Misc counter }
    SelectedSlot: Longint;             { Selected slot to use for this call }

    ClientName  : String;                    { Hostname used by this client }
begin
  {-- Initialize all variables ----------------------------------------------}
  FillChar(Connections, SizeOf(Connections), #0);

  {-- A variable passed to the Accept() call --------------------------------}
  TempLen := SockAddr_len;

  {-- Setup the client address variable -------------------------------------}
  New(ClientAddr);
  FillChar(ClientAddr^, SizeOf(ClientAddr^), #00);

  {-- This loop will keep going till the program is ended -------------------}
  repeat

    {-- Now just wait till someone tries to connect -------------------------}
    repeat
      ClientRC := SockAccept(FtpServerSocket.ServerHandle,
                             ClientAddr,
                             TempLen);

      FtpServerSocket.DoSleep(1);
    until (ClientRC <> -1) OR (TermProgram);

    {-- Select a slot for this session --------------------------------------}
    SelectedSlot := 00;
    for Counter := 01 to FtpMaxSessions do
      if Connections[Counter].Name = '' then
        begin
          SelectedSlot := Counter;
          BREAK;
        end; { if }


    {-- We''ve got an connection, lets initialize it ------------------------}
    if NOT TermProgram then
      begin
        SocksetBlockingIO(ClientRC, false);  { Make that it is non-blocking }

        {-- Lookup host name if allowed, else use the IP address ------------}
        if NOT NoDnsLookup then
          ClientName := SockGetHostNameByAddr(@ClientAddr^.Sin_Addr)
            else ClientName := FtpServerSocket.IpToStr(ClientAddr^.sIn_Addr);

        {-- Make sure we have a slot available for this user ----------------}
        if SelectedSlot > 00 then
          begin
            {-- Initialize all needed data for this session -----------------}
            Connections[SelectedSlot].Name := ClientName;
            Connections[SelectedSlot].IpAddr:= FtpServerSocket.IpToStr(ClientAddr^.sIn_Addr);
            Connections[SelectedSlot].StartTimeStr := TimeStr(true, false);
            Connections[SelectedSlot].ClientRC := ClientRC;

            {-- allocate memory ---------------------------------------------}
            GetMem(Connections[SelectedSlot].srvData, SizeOf(ftpConnectionRec));
            FillChar(Connections[SelectedSlot].srvData^, SizeOf(ftpConnectionRec), #0);

            ftpConnectionRec(Connections[SelectedSlot].SrvData^).UserName := '';
            ftpConnectionRec(Connections[SelectedSlot].SrvData^).PasswordCRC := -1;
            ftpConnectionRec(Connections[SelectedSlot].SrvData^).Validated := FALSE;
            ftpConnectionRec(Connections[SelectedSlot].SrvData^).IsPassive := FALSE;
            ftpConnectionRec(Connections[SelectedSlot].SrvData^).UseDataConn := FALSE;
            ftpConnectionRec(Connections[SelectedSlot].SrvData^).CurArea := -1;
            ftpConnectionRec(Connections[SelectedSlot].SrvData^).CurGroup := -1;
            New(Connections[SelectedSlot].ServerThread, Init);
            ftpConnectionRec(Connections[SelectedSlot].SrvData^).DataThread := TTcpServer.Create;

            {-- Fire up the thread to handle this session -------------------}
            With Connections[SelectedSlot].ServerThread^ do
              CreateThread(StackSize,
                           @Ftp_ExecuteConnection,
                           Pointer(SelectedSlot),
                           0);

            {-- Now notify the system operator ------------------------------}
            RaLog('>', Format('[FTPSERV] [%s] Connection opened', [Connections[SelectedSlot].IpAddr]));
          end
            else begin
                   {-- We cannot accept this session ------------------------}
                   Inc(TotalRefused);
                   SockClose(ClientRc);
                 end; { if busy }
      end; { if not program Terminated }

  until (ClientRC = -1) OR (TermProgram);

  {-- Dispose all allocated memory ------------------------------------------}
  Dispose(ClientAddr);

  ftp_ListenThreadEnded := true;
  ExitThisThread;
end; { func. FtpServerHost }
{$S+}

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure ftp_initserver;
begin
end; { proc. ftp_initserver }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure ftp_SetupServer;
var ErrorStr: String;
begin
  FtpServerSocket := TTcpServer.Create;
  FtpServerSocket.ReUseAddr := true;

  if NOT FtpServerSocket.SetupServer(FtpServerPort, ErrorStr, InAddr_Any) then
    FatalError(ErrorStr);

  FtpServerSocket.Blocking := false;
end; { proc. ftp_SetupServer }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

end. { unit FTPSERV }
