unit SMTPCLI;
(*
**
** Copyright (C) 1999-2003 Maarten Bekers. All rights reserved.
**
** This file is part of EleBBS, EleXer and EleWEB.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*)
{$I COMPILER.INC}
{&Delphi+}
(*
**
** SMTP - Mail sending client component
** Written to follow RFC 821
**
** Copyright (c) 1998-2000 by Maarten Bekers
**
** Created : 23-Apr-2000
** Last update : 23-Apr-2000
**
*)

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)
 INTERFACE
(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

uses TcpCli, Articles;

Const ArticleWaitSecs   = 120;
      KeyboardCheckSecs = 2;

type  ShowLineProc     = procedure(const S: ShortString);

type tSMTPclient = class(tTcpClient)
       protected
        fLastError   : Longint;
        fShowLine    : ShowLineProc;

       public
        fErrorStr    : AnsiString;

        constructor Create;
        destructor  Destroy; override;

        property ShowLine : ShowLineProc READ fShowLine WRITE fShowLine;

        function Quit: Boolean;
        function Helo(Domain: String): Boolean;
        function Mail(MsgFrom: String): Boolean;
        function Rcpt(MsgTo: String): Boolean;
        function Rset: Boolean;
        function Vrfy(Mailbox: String): Boolean;
        function Noop: Boolean;
        function Data(var MsgText: MessageTextType): Boolean;


        procedure ShowAllText(Require: Boolean);
        function CheckError(TestSet: Array of Word; var LastErr: Longint): Boolean;
     end; { tSMTP client }


(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)
 IMPLEMENTATION
(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)
 uses ApTimer, LongStr, WordStr, SysUtils,
       SockDef, SockFunc, StUtils, Global;
(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

constructor tSMTPClient.Create;
begin
  inherited Create;

  fErrorStr:= '';
end; { constructor Create }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

destructor tSMTPClient.Destroy;
begin
  inherited Destroy;
end; { destructor Destroy }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

procedure tSMTPClient.ShowAllText(Require: Boolean);
var RecvStr  : AnsiString;
    Count    : Byte;
begin
  if Require then
    begin
      Count := 40;

      While (Count > 0) AND (NOT DataAvailable) AND (ConnectionAlive) do
        begin
          DoSleep(100);

          Dec(Count);
        end; { while }
    end; { require }

  while (DataAvailable) AND (ConnectionAlive) do
    begin
      RecvStrLn(RecvStr, false);

      if Assigned(fShowLine) then
        fShowLine(RecvStr);

      if Require then BREAK;
    end; { while }

end; { proc. ShowAllText }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSMTPClient.Quit: Boolean;
begin
  SendStrLn('QUIT');

  Result := CheckError([221], fLastError);
  ShowAllText(false);
end; { func. Quit }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSMTPClient.Helo(Domain: String): Boolean;
begin
  SendStrLn('HELO ' + Domain);

  Result := CheckError([250], fLastError);
  ShowAllText(false);
end; { func. Helo }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSmtpClient.Mail(MsgFrom: String): Boolean;
begin
  SendStrLn('MAIL FROM:<' + MsgFrom + '>');

  Result := CheckError([250], fLastError);
  ShowAllText(false);
end; { func. Mail }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSmtpClient.Vrfy(Mailbox: String): Boolean;
begin
  SendStrLn('VRFY ' + MailBox);

  Result := CheckError([250, 251], fLastError);
  ShowAllText(false);
end; { func. Vrfy }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSmtpClient.Rcpt(MsgTo: String): Boolean;
begin
  SendStrLn('RCPT TO:<' + Trim(MsgTo) + '>');

  Result := CheckError([250, 251], fLastError);
  ShowAllText(false);
end; { func. Rcpt }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSmtpClient.Rset: Boolean;
begin
  SendStrLn('RSET');

  Result := CheckError([250], fLastError);
  ShowAllText(false);
end; { func. Rset }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSmtpClient.Noop: Boolean;
begin
  SendStrLn('NOOP');

  Result := CheckError([250], fLastError);
  ShowAllText(false);
end; { func. Noop }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSmtpClient.Data(var MsgText: MessageTextType): Boolean;
var TempStr: AnsiString;
    TxtPtr : Longint;
    CheckKb: EventTimer;
begin
  Result := true;
  if DataAvailable then FlushData;       { Make sure previous data is flushed }

  {-- Send command to initiate message transfer ----------------------------}
  SendStrLn('DATA');
  if NOT CheckError([354], fLastError) then
    begin
      Result := false;
      EXIT;
    end; { if }

  {-- Initialize some timers before sending --------------------------------}
  TxtPtr := 00;
  TempStr := '';

  NewTimer(CheckKB, Secs2Tics(KeyboardCheckSecs));

  {-- Now actually send the message ----------------------------------------}
  While (TxtPtr <= (StrLen(MsgText) + 01)) do
    begin
      if (NOT (MsgText[TxtPtr] in [#13, #10])) then
        TempStr := TempStr + MsgText[TxtPtr]
          else begin
                 Inc(TxtPtr);

                 if TempStr = '.' then TempStr := '..';
                   SendStrLn(TempStr);

                 TempStr := '';

                 if TimerExpired(CheckKB) then
                   begin
                     NewTimer(CheckKB, Secs2Tics(KeyboardCheckSecs));

                     if NOT ConnectionAlive then BREAK;
                   end; { if }
               end; { if }

      Inc(TxtPtr);
    end; { while }

  {-- End the message ------------------------------------------------------}
  SendStrLn('.');
  if CheckError([250], fLastError) then
    begin
      Result := true;
      { Article posted correctly }
    end
      else begin
             Result := false;
           end; { not posted }
end; { func. Data }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

function tSMTPClient.CheckError(TestSet: Array of Word; var LastErr: Longint): Boolean;
var Counter       : Longint;
    FoundResponse : Longint;
begin
  Result := false;
  if NOT ConnectionAlive then EXIT;
  WaitForData;

  RecvStrLn(fErrorStr, false);

  FoundResponse := FVal(ExtractWord(fErrorStr, 1, defExtractWord, false, false));

  for Counter := 00 to High(TestSet) do
    if FoundResponse = TestSet[Counter] then
     begin
       LastErr := TestSet[Counter];
       Result := true;
       EXIT;
     end; { CheckError }

end; { func. CheckError }

(*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-+-*-*)

end. { smtpcli }
